package com.huiju.welcome.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.welcome.common.base.BaseController;
import com.huiju.welcome.common.base.ResponseBean;
import com.huiju.welcome.model.TaNotice;
import com.huiju.welcome.service.ITaNoticeService;
import com.huiju.welcome.utils.StatusUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author jobob
 * @since 2019-03-12
 */
@RestController
@RequestMapping("/")
public class TaNoticeController extends BaseController {
    @Autowired
    private ITaNoticeService taNoticeService;

    @ApiOperation(value = "公告新增", notes = "公告新增")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "title", value = "标题"),
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "content", value = "内容"),
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "publisher", value = "发布人"),
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "publishDate", value = "发布时间")
    })
    @RequestMapping(value = "/notice", method = RequestMethod.POST)
    public ResponseBean create(@RequestBody TaNotice taNotice, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        taNotice.setStatus(StatusUtils.Normal);
        taNotice.setCreateDate(LocalDateTime.now());

        boolean success = taNoticeService.save(taNotice);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "新增内容失败, 请重试");
            return responseBean;
        }

        responseBean.addSuccess(taNotice);
        return responseBean;
    }

    @ApiOperation(value = "公告编辑", notes = "公告编辑")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "title", value = "标题"),
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "content", value = "内容"),
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "publisher", value = "发布人"),
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "publishDate", value = "发布时间"),
            @ApiImplicitParam(paramType = "body", dataType = "Integer", name = "status", value = "状态")
    })
    @RequestMapping(value = "/notice", method = RequestMethod.PUT)
    public ResponseBean update(@RequestBody TaNotice taNotice, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        boolean success = taNoticeService.updateById(taNotice);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "更新内容失败, 请重试");
            return responseBean;
        }

        responseBean.addSuccess(taNotice);
        return responseBean;
    }

    @ApiOperation(value = "公告删除", notes = "公告删除")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataType = "Integer", name = "id", value = "内容ID")
    })
    @RequestMapping(value = "/notice/{id}", method = RequestMethod.DELETE)
    public ResponseBean delete(@PathVariable int id, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        TaNotice taNotice = taNoticeService.getById(id);
        taNotice.setStatus(StatusUtils.Delete);

        boolean success = taNoticeService.updateById(taNotice);
        if (!success) {
            responseBean.addError(HttpStatus.SC_INTERNAL_SERVER_ERROR, "删除内容失败, 请重试");
            return responseBean;
        }

        return responseBean;
    }

    @ApiOperation(value = "获取公告详情", notes = "获取公告详情")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataType = "Integer", name = "id", value = "公告ID")
    })
    @RequestMapping(value = "/notice/{id}", method = RequestMethod.GET)
    public ResponseBean get(@PathVariable int id, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        TaNotice taNotice = taNoticeService.getById(id);
        if (null == taNotice || taNotice.getStatus().intValue() == StatusUtils.Delete) {
            responseBean.addError(HttpStatus.SC_BAD_REQUEST, "没有找到该ID对应数据");
            return responseBean;
        }

        responseBean.addSuccess(taNotice);
        return responseBean;
    }


    @ApiOperation(value = "获取展示列表", notes = "获取展示列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageNum", value = "当前页"),
            @ApiImplicitParam(paramType = "query", dataType = "int", name = "pageSize", value = "每页数量"),
    })
    @RequestMapping(value = "/notice", method = RequestMethod.GET)
    public ResponseBean list(@RequestParam Integer pageNum, @RequestParam  Integer pageSize, HttpServletRequest request) {
        ResponseBean responseBean = new ResponseBean();

        Page<TaNotice> pg = new Page<>();
        pg.setCurrent(pageNum);
        pg.setSize(pageSize);
        QueryWrapper<TaNotice> queryWrapper = new QueryWrapper<>();
        queryWrapper.orderByDesc("weight");
        queryWrapper.eq("status", StatusUtils.Normal);
        responseBean.addSuccess(taNoticeService.page(pg, queryWrapper));

        return responseBean;
    }

}
