package com.huiju.welcome.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.welcome.common.WebSocketServer;
import com.huiju.welcome.mapper.*;
import com.huiju.welcome.model.*;
import com.huiju.welcome.service.*;
import com.huiju.welcome.utils.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-06-12
 */
@Service
public class TaCustomerServiceImpl extends ServiceImpl<TaCustomerMapper, TaCustomer> implements ITaCustomerService {
    Logger log = LoggerFactory.getLogger(TaCustomerServiceImpl.class);

    @Autowired
    TaCustomerMapper taCustomerMapper;

    @Autowired
    TaCustomerMapMapper taCustomerMapMapper;

    @Autowired
    TaMainUsherRecordMapper taMainUsherRecordMapper;

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    ITaMainUsherRecordService iTaMainUsherRecordService;

    @Autowired
    ITaFirstUsherRecordService iTaFirstUsherRecordService;

    @Autowired
    ITaVisitorAppointmentService iTaVisitorAppointmentService;

    @Autowired
    ITaFollowupRecordService iTaFollowupRecordService;

    @Autowired
    TaWechatUserMapper taWechatUserMapper;

    @Autowired
    SysUserMapper sysUserMapper;

    @Autowired
    ISysParamService iSysParamService;

    @Autowired
    ISysDictService iSysDictService;

    @Autowired
    MiniApp miniApp;

    @Autowired
    ICameraService cameraService;

    @Autowired
    SMSUtils smsUtils;

    @Autowired
    IMessageService iMessageService;

    @Override
    public List<TaPerson> getGroup(Integer customerId) {
        return taCustomerMapMapper.getGroup(customerId);
    }

    @Override
    public Map<String, Object> getLastVisitInfo(Integer customerId) {
        Map<String, Object> summaryOfCustomer = taMainUsherRecordMapper.getSummaryOfCustomer(customerId);

        Object o = summaryOfCustomer.get("visiteDate");
        if (null != o) {
            String visiteDate = summaryOfCustomer.get("visiteDate").toString().replace(".0", "");
            DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

            summaryOfCustomer.put("visiteDate", LocalDateTime.parse(visiteDate,df));
        }

        return summaryOfCustomer;
    }

    @Override
    public void judgeNewCust(TaPerson taPerson) throws Exception {
        // 首先判定这个人 2 小时内是否来访过
        Float interval = iSysParamService.getCustomerVisitInterval();
        LocalDateTime dt = LocalDateTime.now().minusMinutes(interval.intValue());
        List<TaMainUsherRecord> lastList = taMainUsherRecordMapper.getLastList(taPerson.getRealId(), dt);
        if (null != lastList && lastList.size() > 0) return;

        // 查询是否有映射客户 - 如果有多个映射客户, 先随机取一条
        // 暂时没有找到好的替代方案
        TaCustomer taCustomer = taCustomerMapper.getCustomerOf(taPerson.getPersonId(), taPerson.getRealId());

        // 如果是新人过来
        if (null == taCustomer) {
//            taCustomer = newCustomerByPerson(taPerson);
        } else {
            iMessageService.toggledByCustomer(taCustomer);
        }

        // 新增前台记录
//        iTaMainUsherRecordService.newRecordByDeviceDeprecated(taCustomer, taPerson);
        iTaMainUsherRecordService.newRecordByDevice(taPerson, taCustomer);
        return;
    }

    @Override
    public void deleteCustOfPerson(Integer personId, Integer realId) {
        // 删除映射表
        QueryWrapper<TaCustomerMap> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("person_id", personId).or().eq("person_id", realId);
        taCustomerMapMapper.delete(queryWrapper);

        // 删除客户表, 更新状态为删除
        // 默认客户ID 与人员ID是一致的
        UpdateWrapper<TaCustomer> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("status", StatusUtils.Delete);
        updateWrapper.eq("customer_id", personId).or().eq("customer_id", realId);
        taCustomerMapper.update(new TaCustomer(), updateWrapper);
    }

    @Override
    public List<TaCustomer> getSomeCustMayBe(String plateNumber) {
        return taCustomerMapper.getSomeCustMayBe(plateNumber, null);
    }

    @Override
    public TaCustomer newCustByCarDevice(TaCarRecord taCarRecord, TaVisitorAppointment ...taVisitorAppointments) {
        TaCustomer taCustomer = new TaCustomer();
        TaVisitorAppointment appointment = null != taVisitorAppointments && taVisitorAppointments.length > 0 ? taVisitorAppointments[0] : null;

        String name = StringUtils.ifNull(taCarRecord.getCustomerName(), null != appointment ? appointment.getPersonName() : null);
        String phone = null != appointment ? appointment.getPhone() : null;
        Integer consultantId = null != appointment ? appointment.getCreateUser() : null;
        SysUser sysUser = null != consultantId ? sysUserMapper.selectById(consultantId) : null;

        taCustomer.setCreateDate(LocalDateTime.now());
        taCustomer.setStatus(StatusUtils.Normal);
        taCustomer.setCustomerName(name);
        taCustomer.setPhone(phone);
        taCustomer.setAvatar(taCarRecord.getDriverImage());
        taCustomer.setCarModel(taCarRecord.getCarModel());
        taCustomer.setPlateNumber(taCarRecord.getPlateNumber());
        taCustomer.setConsultantId(consultantId);
        taCustomer.setSex(null != appointment ? appointment.getSex() : null);
        if (null != consultantId){
            SysUser usr = sysUserMapper.selectById(consultantId);
            if (null != usr){
                taCustomer.setRealtyConsultant(usr.getUsername());
            }
        }
        if (null != sysUser) {
            if (CommConstant.IDENTITY_CHANNEL_MANAGER.equals(sysUser.getUserIdentity())) {
                taCustomer.setCustomerType(CommConstant.CUSTOMER_OF_CHANNEL);
            }else if(CommConstant.IDENTITY_PLANNING_MANAGER.equals(sysUser.getUserIdentity())){
                taCustomer.setCustomerType(CommConstant.CUSTOMER_OF_PLANNING);
            }
        }
        
        TaPerson taPerson = iTaPersonService.newPersonByFront(taCustomer);
        taCustomer.setCustomerId(taPerson.getRealId());

        TaCustomerMap customerMap = new TaCustomerMap();
        customerMap.setCustomerId(taCustomer.getCustomerId());
        customerMap.setPersonId(taPerson.getPersonId());

        taCustomerMapMapper.insert(customerMap);

        insertCustomer(taCustomer);

        return taCustomer;
    }

    @Override
    public void notifyConsultant(TaCustomer taCustomer) throws Exception {
        String tpl = miniApp.getTpls().getVisitor();
        String page = CommConstant.PAGE_MY_CUSTOMER + "?id=" + taCustomer.getCustomerId();
        String openid = taWechatUserMapper.getOpenidOf(taCustomer.getConsultantId(), CommConstant.WECHAT_CLIENT_MINIAPP);
        if (null == openid) {
            throw new Exception("发送消息给置业失败, 当前置业[" + taCustomer.getConsultantId() + "]未绑定小程序账号");
        }
        List<Object> data = new ArrayList<Object>(){{
            add(taCustomer.getCustomerName());
            add(taCustomer.getPhone());
            add("您有客户来访, 请及时接待!");
        }};

        if (miniApp.sendTPLMessage(tpl, openid, page, data).getCode() == -1) {
            // 没有 formid 发短信
            String phone = sysUserMapper.getPhoneByOpenid(openid);
            if (null != phone) {
                smsUtils.sendVisitor(phone, null == taCustomer ? "": StringUtils.ifNull(taCustomer.getCustomerName(), ""));
            }
        }
    }
    
    /**
     * 推送车辆信息给置业经理
     * @param taFirstUsherRecord
     * @param taCustomer
     * @throws Exception
     */
    @Override
    public void notifyFirstUsherConsultantManager(TaFirstUsherRecord taFirstUsherRecord, TaCustomer taCustomer) {
        List<SysUser> sysUserList = sysUserMapper.filterByIdentity(CommConstant.IDENTITY_CONSULTANT_MANAGER);
        String message ="";
        if (null != taCustomer){
            if (null == taCustomer.getConsultantId()){
                message = "请安排置业接待。";
            }else{
                SysUser sysUser = sysUserMapper.selectById(taCustomer.getConsultantId());
                message = "置业"+sysUser.getUsername()+"客户，请安排接待";
            }
        }else {
            message = "客户已到达, 请及时关注。";
        }
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
        
        String visitData = formatter.format(taFirstUsherRecord.getVisiteDate());
        List<Object> data = new ArrayList<Object>();
        data.add(visitData);
        data.add(taFirstUsherRecord.getPlateNumber()+" "+ taFirstUsherRecord.getColor() + " " + taFirstUsherRecord.getCarModel());
        data.add(message);
        sysUserList.stream().forEach(e -> {
            String tpl = miniApp.getTpls().getFirstUsher();
            String page = CommConstant.PAGE_MY_FIRSTUSHER + "?id=" + taFirstUsherRecord.getRecId();
            String openid = e.getOpenid();
            if (null == openid) {
                return;
            }
//            miniApp.sendTPLMessage(tpl, openid, page, data);
            if (miniApp.sendTPLMessage(tpl, openid, page, data).getCode() == -1) {
                // 没有 formid 发短信
                String phone = sysUserMapper.getPhoneByOpenid(openid);
                if (null != phone) {
                    smsUtils.sendVisitor(phone, null == taCustomer ? "": StringUtils.ifNull(taCustomer.getCustomerName(), ""));
                }
            }
        });
    }
    
    @Override
    public TaCustomer getCutomerByPhoneAndPlatenumber(String phone, String plateNumber) {
        List<TaCustomer> list = taCustomerMapper.getSomeCustMayBe(plateNumber,phone);
        if (list.size() > 0){
            return list.get(0);
        }
        return null;
    }
    
    @Override
    public boolean updateConsultant(TaCustomer taCustomer, Integer consultantId) {
        SysUser usr = sysUserMapper.selectById(consultantId);
        if (null == usr) {
            log.error("更新客户置业失败, 置业ID {} 不存在", consultantId);
            return false;
        }

        Integer originConsultant = taCustomer.getConsultantId();
        taCustomer.setConsultantId(consultantId);
        taCustomer.setRealtyConsultant(usr.getUsername());

        // 如果变更了置业顾问
        if (null == originConsultant || originConsultant.intValue() < 1 || !originConsultant.equals(consultantId)) {
            try {
                notifyConsultant(taCustomer);
            } catch (Exception e) {}
        }

        int row = taCustomerMapper.updateById(taCustomer);
        return row > 0;
    }

    @Override
    public TaCustomer newCustByMainUsher(TaMainUsherRecord taMainUsherRecord, boolean notify) throws Exception {
        Integer personId = taMainUsherRecord.getPersonId();
        if (null == personId) {
            TaPerson taPerson = iTaPersonService.newPersonByUsher(taMainUsherRecord);
            personId = taPerson.getRealId();
            taMainUsherRecord.setPersonId(personId);
        }
        
        Integer custId = taMainUsherRecord.getCustomerId();
        TaCustomer cust;
        if (null == custId) {
            cust = copyDataFromMainUsher(taMainUsherRecord);
            if (!insertCustomer(cust)) {
                throw new Exception("新增客户信息失败");
            }
            
            taMainUsherRecord.setCustomerId(cust.getCustomerId());
        } else {
            cust = taCustomerMapper.selectById(custId);
            cust.setReceiver(taMainUsherRecord.getReceiver());
            taCustomerMapper.updateById(cust);
        }

        TaCustomerMap taCustomerMap = taCustomerMapMapper.getCustomerOf(personId);
        if (null == taCustomerMap) {
            TaCustomerMap custMap = new TaCustomerMap();
            custMap.setCustomerId(cust.getCustomerId());
            custMap.setPersonId(taMainUsherRecord.getPersonId());
            
            int row = taCustomerMapMapper.insert(custMap);
            if (row <= 0) {
                throw new Exception("生成置业客户映射失败");
            }
        }

        // 发消息给置业
        if (notify){
            //通知前端
            sendMessageAll(null,cust,taMainUsherRecord);
            //推送置业经理
//            notifyManager(cust);
            iMessageService.toggledByCustomer(cust);
        }
        
        return cust;
    }
    
    @Override
    public TaCustomer newCustByFirstUsher(TaFirstUsherRecord taFirstUsherRecord, boolean notify) throws Exception {
        Integer personId = taFirstUsherRecord.getPersonId();
        if (null == personId) {
            TaPerson taPerson = iTaPersonService.newPersonByFirstUsher(taFirstUsherRecord);
            personId = taPerson.getRealId();
            taFirstUsherRecord.setPersonId(personId);
        }
        
        TaCustomer cust = copyDataFromFirstUsher(taFirstUsherRecord);
        if (!insertCustomer(cust)) {
            throw new Exception("新增客户信息失败");
        }
        
        TaCustomerMap taCustomerMap = taCustomerMapMapper.getCustomerOf(personId);
        if (null == taCustomerMap) {
            TaCustomerMap custMap = new TaCustomerMap();
            custMap.setCustomerId(cust.getCustomerId());
            custMap.setPersonId(taFirstUsherRecord.getPersonId());
            
            int row = taCustomerMapMapper.insert(custMap);
            if (row <= 0) {
                throw new Exception("生成置业客户映射失败");
            }
        }
        
        // 发消息给置业
        if (notify){
            //通知吧台来人了
            sendMessageAll(taFirstUsherRecord,cust,null);

            //通知吧台人数等信息
//            sendMessageToBar(cust,taFirstUsherRecord.getPersonNum());

            //推送置业经理
//            notifyManager(cust);
            iMessageService.toggledByCustomer(cust);
        }
        
        return cust;
    }
    
    private TaCustomer copyDataFromFirstUsher(TaFirstUsherRecord taFirstUsherRecord) {
        TaCustomer taCustomer = new TaCustomer();
        taCustomer.setCustomerId(taFirstUsherRecord.getPersonId());       // 默认客户ID与人员ID一致
        taCustomer.setCustomerName(taFirstUsherRecord.getPersonName());
        taCustomer.setPhone(taFirstUsherRecord.getPhone());
        taCustomer.setPlateNumber(taFirstUsherRecord.getPlateNumber());
        taCustomer.setCarModel(taFirstUsherRecord.getCarModel());
        taCustomer.setChannel(taFirstUsherRecord.getChannel());
        taCustomer.setPurpose(taFirstUsherRecord.getPurpose());
        taCustomer.setStatus(StatusUtils.Normal);
        taCustomer.setCreateDate(LocalDateTime.now());
        taCustomer.setConsultantId(taFirstUsherRecord.getConsultantId());
        taCustomer.setRealtyConsultant(taFirstUsherRecord.getRealtyConsultant());
    
        return taCustomer;
    }
    
    @Override
    public Map<String, Object> getHistoryRecBy(Map<String, Object> filters) {
        String phone = filters.get("phone").toString();
        Map<String, Object> result = new HashMap<String, Object>(){{
            put("phone", phone);
        }};

        List<TaVisitorAppointment> appointments = iTaVisitorAppointmentService.getAppointmentsBy(filters);
        if (null != appointments && appointments.size() > 0) {
            result.put("source", "appointment");
            result.put("appointment", 1);
            result.put("name", appointments.get(0).getPersonName());
            result.put("consultantId", appointments.get(0).getCreateUser());
            SysUser user= sysUserMapper.selectById(appointments.get(0).getCreateUser());
            result.put("consultantName", user.getUsername());
            result.put("sex", appointments.get(0).getSex());
            result.put("plateNumber", appointments.get(0).getPlatNumber());
            result.put("customerId", "");
        }

        List<TaCustomer> custs = taCustomerMapper.getSomeCustMayBe(null, phone);
        if (null != custs && custs.size() > 0) {
            result.put("source", "customer");
            result.put("name", custs.get(0).getCustomerName());
            result.put("consultantId", custs.get(0).getConsultantId());
            result.put("consultantName", custs.get(0).getRealtyConsultant());
            result.put("purpose", custs.get(0).getPurpose());
            result.put("sex", custs.get(0).getSex());
            result.put("plateNumber", custs.get(0).getPlateNumber());
            result.put("customerId", custs.get(0).getCustomerId());
        }

        // 如果车牌不存在, 查询车辆来访记录
        // 只使用最近的一次记录
        if (null == result.get("plateNumber")) {
            QueryWrapper<TaFirstUsherRecord> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("phone", phone);
            queryWrapper.ge("status", StatusUtils.Ready);
            queryWrapper.orderByDesc("visite_date");
            queryWrapper.last("limit 1");

            TaFirstUsherRecord taFirstUsherRecord = iTaFirstUsherRecordService.getOne(queryWrapper);
            if (null != taFirstUsherRecord && null != taFirstUsherRecord.getPlateNumber()) {
                result.put("plateNumber", taFirstUsherRecord.getPlateNumber());
            }
        }

      return result;
    }

    @Override
    public TaCustomer getSameByPerson(Integer personId) {
        return taCustomerMapper.getCustomerOf(personId, personId);
    }

    @Override
    public boolean updataByCarRecord(TaCustomer taCustomer, TaCarRecord taCarRecord) {
        boolean needUpate = false;

        if (StringUtils.isEmpty(taCustomer.getPlateNumber())) {
            needUpate = true;
            taCustomer.setPlateNumber(taCarRecord.getPlateNumber());
            taCustomer.setCarModel(taCarRecord.getCarModel());
        }

        if (needUpate) {
            return taCustomerMapper.updateById(taCustomer) > 0;
        }

        return true;
    }

    @Override
    public TaCustomer getByPhoneOrPlate(String phone, String plateNumber) {
        QueryWrapper<TaCustomer> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(phone != null,"phone", phone).or().eq(plateNumber != null, "plate_number", plateNumber);
        // 故意设置一个查询不到数据的条件
        queryWrapper.eq(null == phone || null == plateNumber, "customer_id", -1);
        queryWrapper.orderByDesc("status");
        queryWrapper.orderByDesc("create_date");
        queryWrapper.last("limit 1");

        return taCustomerMapper.selectOne(queryWrapper);
    }

    @Override
    public TaCustomer getWithLastVisit(Integer id) {
        TaCustomer taCustomer = taCustomerMapper.selectById(id);
        if (null == taCustomer) {
            return null;
        }
        //获取字典表所有信息
        List<SysDict> dictList = iSysDictService.list();
        dictList.stream().forEach(e -> {
            //对比数据重新赛值
            if (null != taCustomer.getNowArea() && e.getId().equals(taCustomer.getNowArea())){
                taCustomer.setNowAreaLable(e.getLabel());
            }
            if (null != taCustomer.getFamilyWork() && e.getId().equals(taCustomer.getFamilyWork())){
                taCustomer.setFamilyWorkLable(e.getLabel());
            }
            if (null != taCustomer.getFamilyIndustry() && e.getId().equals(Integer.valueOf(taCustomer.getFamilyIndustry()))){
                taCustomer.setFamilyIndustryLable(e.getLabel());
            }
            if (null != taCustomer.getFamilyDuty() && e.getId().equals(Integer.valueOf(taCustomer.getFamilyDuty()))){
                taCustomer.setFamilyDutyLable(e.getLabel());
            }
            if (null != taCustomer.getHousePurchasing() && e.getId().equals(taCustomer.getHousePurchasing())){
                taCustomer.setHousePurchasingLable(e.getLabel());
            }
            if (null != taCustomer.getHouseProperty() && e.getId().equals(taCustomer.getHouseProperty())){
                taCustomer.setHousePropertyLable(e.getLabel());
            }
            if (null != taCustomer.getHouseLivingArea() && e.getId().equals(taCustomer.getHouseLivingArea())){
                taCustomer.setHouseLivingAreaLable(e.getLabel());
            }
            if (null != taCustomer.getHouseType() && e.getId().equals(taCustomer.getHouseType())){
                taCustomer.setHouseTypeLable(e.getLabel());
            }
            if (null != taCustomer.getFamilyStructure() && e.getId().equals(taCustomer.getFamilyStructure())){
                taCustomer.setFamilyStructureLable(e.getLabel());
            }
            if (null != taCustomer.getCustomerAge() && e.getId().equals(taCustomer.getCustomerAge())){
                taCustomer.setCustomerAgeLable(e.getLabel());
            }
            if (null != taCustomer.getFavorLevel() && e.getId().equals(Integer.valueOf(taCustomer.getFavorLevel()))){
                taCustomer.setFavorLevelLable(e.getLabel());
            }
        });

        TaMainUsherRecord taMainUsherRecord = iTaMainUsherRecordService.getTodayRecord(taCustomer.getCustomerId());
        if (null != taMainUsherRecord) {
            List<TaFollowupRecord> follows = iTaFollowupRecordService.getByMainRecord(taMainUsherRecord.getRecId());

            if (null != follows && follows.size() > 0) {
                Map<String, Object> lastVisit = new HashMap<String, Object>(){{
                    put("follows", follows);
                }};
                taCustomer.setLastVisit(lastVisit);
            }
        }

        return taCustomer;
    }

    @Override
    public IPage<TaCustomer> getCustomerListOfFront(int pageNumber, int pageSize, Integer consultantId, String nameOrPhoneOrReceiver, String identity, Integer consultantType,String customerType) {
        IPage<TaCustomer> page = new Page<>(pageNumber, pageSize);

        String identityParams = StringUtils.isEmpty(identity) ? null : identity;

        return taCustomerMapper.getCustomerListOfFront(page, consultantId, nameOrPhoneOrReceiver, identityParams, consultantType,customerType);
    }

    @Override
    public List<CustomerExcel> getCustomersEqualCustomerPage(String customerName, String phone, String platNumber, Integer agentUserId) {
        List<CustomerExcel> result = new ArrayList<>();
        List<TaCustomer> customers = taCustomerMapper.getCustomersEqualCustomerPage(customerName, phone, platNumber, agentUserId);
        List<SysDict> dict = iSysDictService.list();

        if (null != customers) {
            for (int i = 0; i < customers.size(); i ++) {
                TaCustomer customer = customers.get(i);

                CustomerExcel item = new CustomerExcel();
                item.setCustomerId(customer.getCustomerId());
                item.setCustomerName(customer.getCustomerName());
                item.setSex(null == customer.getSex() ? "未知" : (2 == customer.getSex() ? "女" : "男"));
                item.setCustomerAge(getItemNameOfDict(dict, customer.getCustomerAge()));
                item.setPhone(customer.getPhone());
                item.setIdcard(customer.getIdcard());
                item.setPlateNumber(customer.getPlateNumber());
                item.setCarModel(customer.getCarModel());
                item.setConsultant(customer.getRealtyConsultant());
                item.setChannel("1".equals(customer.getChannel()) ? "预约": "未约");
                item.setPurpose("1".equals(customer.getPurpose()) ? "看房": "其他");
                item.setHaveHouse(null != customer.getHaveHouse() && 1 == customer.getHaveHouse() ? "是" : "否");
                item.setOldCommunity(customer.getOldCommunity());
                item.setNowArea(getItemNameOfDict(dict, customer.getNowArea()));
                item.setNowCity(customer.getNowCity());
                item.setNowCommunity(customer.getNowCommunity());
                item.setFamilyWork(getItemNameOfDict(dict, customer.getFamilyWork()));
                item.setCityName(customer.getCityName());
                item.setFamilyCompany(customer.getFamilyCompany());
                item.setFamilyIndustry(customer.getFamilyIndustry());
                item.setFamilyDuty(customer.getFamilyDuty());
                item.setHousePurchasing(getItemNameOfDict(dict, customer.getHousePurchasing()));
                item.setHouseProperty(getItemNameOfDict(dict, customer.getHouseProperty()));
                item.setHouseLivingArea(getItemNameOfDict(dict, customer.getHouseLivingArea()));
                item.setHouseType(getItemNameOfDict(dict, customer.getHouseType()));
                item.setFamilyStructure(getItemNameOfDict(dict, customer.getFamilyStructure()));
                item.setFavorLevel(getItemNameOfDict(dict, null == customer.getFavorLevel() ? null : Integer.valueOf(customer.getFavorLevel())));
                item.setRemark(customer.getRemark());

                result.add(item);
            }
        }

        return result;
    }


    private String getItemNameOfDict(List<SysDict> dict, Integer id) {
        if (null == dict || null == id) return null;

        for (int i = 0; i < dict.size(); i ++) {
            if (id == dict.get(i).getId()) {
                return dict.get(i).getLabel();
            }
        }

        return null;
    }

    @Override
    public IPage<TaCustomer> getAttributionList(int pageNumber, int pageSize) {
        IPage<TaCustomer> page = new Page<>(pageNumber, pageSize);
        return taCustomerMapper.getAttributionList(page);
    }
    
    @Override
    public IPage<TaCustomer> getUnattributionList(int pageNumber, int pageSize) {
        IPage<TaCustomer> page = new Page<>(pageNumber, pageSize);
        return taCustomerMapper.getUnattributionList(page);
    }

    @Override
    public boolean
    updateByMainUsher(Integer customerId, JSONObject params) {
        if (null == customerId || customerId == 0) {
            return false;
        }

        Integer recId = params.getInteger("recId");
        if (null == recId || recId == 0) {
            return false;
        }

        TaCustomer customer = taCustomerMapper.selectById(customerId);
        if (null == customer) {
            return false;
        }

        TaMainUsherRecord taMainUsherRecord = taMainUsherRecordMapper.selectById(recId);
        if (null == taMainUsherRecord) {
            return false;
        }
        
        //原本有人脸先删在增
        if (!StringUtils.isEmpty(customer.getAvatar())){
            //阿里云删除人脸
            cameraService.deleteFace(taMainUsherRecord.getPersonId());
        }
        
        if (StringUtils.isEmpty(params.getString("avatar"))) {
            customer.setAvatar(params.getString("avatar"));
            taMainUsherRecord.setAvatar(params.getString("avatar"));
            //解除人脸关系
            QueryWrapper<TaCustomerMap> customerMapQueryWrapper = new QueryWrapper<>();
            customerMapQueryWrapper.eq("person_id",taMainUsherRecord.getPersonId());
            taCustomerMapMapper.delete(customerMapQueryWrapper);
            taMainUsherRecord.setPersonId(null);
        }else {
            customer.setAvatar(params.getString("avatar"));
            taMainUsherRecord.setAvatar(params.getString("avatar"));
            // 修改人脸并下发
            TaPerson taPerson = iTaPersonService.getByIdOrReal(taMainUsherRecord.getPersonId());
            if (null == taPerson) {
                return false;
            }
            taPerson.setAvatar(params.getString("avatar"));
            iTaPersonService.updateById(taPerson);
            cameraService.dispatchPerson(taPerson);
        }
    
        customer.setPlateNumber(params.getString("plateNumber"));
        customer.setCarModel(params.getString("carModel"));
        customer.setCarColor(params.getString("carColor"));
        customer.setCarImage(params.getString("carImage"));
        taMainUsherRecord.setPlateNumber(params.getString("plateNumber"));
        taMainUsherRecord.setCarModel(params.getString("carModel"));
        taMainUsherRecord.setColor(params.getString("carColor"));
        taMainUsherRecord.setCarImage(params.getString("carImage"));
    
        customer.setReceiver(params.getString("receiver"));
        taMainUsherRecord.setReceiver(params.getString("receiver"));

        
        boolean a = taCustomerMapper.updateById(customer) > 0;
        boolean b = taMainUsherRecordMapper.updateById(taMainUsherRecord) > 0;

        if (null != params.getInteger("fromRecId") && !params.getInteger("fromRecId").equals(recId)){
            TaMainUsherRecord taMainUsherRecordFrom = new TaMainUsherRecord();
            taMainUsherRecordFrom.setRecId(params.getInteger("fromRecId"));
            taMainUsherRecordFrom.setStatus(CommConstant.INVALID_STATUS);
            taMainUsherRecordMapper.updateById(taMainUsherRecordFrom);
        }
        
        return a && b;
    }

    private TaCustomer newCustomerByPerson(TaPerson taPerson) throws Exception {
        TaCustomer taCustomer = new TaCustomer();
        taCustomer.setCustomerId(taPerson.getRealId());
        taCustomer.setAvatar(taPerson.getAvatar());
        taCustomer.setPhone(taPerson.getPhone());
//            taCustomer.setSex(taPerson.getSex());
        taCustomer.setCustomerName(taPerson.getName());
        taCustomer.setCreateDate(LocalDateTime.now());
        taCustomer.setStatus(StatusUtils.Normal);

        if (!insertCustomer(taCustomer)) {
            throw new Exception("新增客户失败");
        }

        int mapsNum = taPerson.getPersonId().equals(taPerson.getRealId()) ? 1 : 2;

        for (int i = 0; i < mapsNum; i ++) {
            TaCustomerMap custMap = new TaCustomerMap();
            custMap.setPersonId(i == 0 ? taPerson.getRealId() : taPerson.getPersonId());
            custMap.setCustomerId(taCustomer.getCustomerId());
            int row = taCustomerMapMapper.insert(custMap);
            if (row < 1) {
                throw new Exception("新增客户映射信息失败");
            }
        }

        return taCustomer;
    }

    private TaCustomer copyDataFromMainUsher(TaMainUsherRecord taMainUsherRecord) {
        TaCustomer taCustomer = new TaCustomer();
        taCustomer.setCustomerId(taMainUsherRecord.getPersonId());       // 默认客户ID与人员ID一致
        taCustomer.setCustomerName(taMainUsherRecord.getPersonName());
        taCustomer.setSex(taMainUsherRecord.getSex());
        taCustomer.setPhone(taMainUsherRecord.getPhone());
        taCustomer.setAvatar(taMainUsherRecord.getAvatar());
        taCustomer.setPlateNumber(taMainUsherRecord.getPlateNumber());
        taCustomer.setCarModel(taMainUsherRecord.getCarModel());
        taCustomer.setChannel(taMainUsherRecord.getChannel());
        taCustomer.setPurpose(taMainUsherRecord.getPurpose());
        taCustomer.setConsultantId(taMainUsherRecord.getConsultantId());
        taCustomer.setRealtyConsultant(taMainUsherRecord.getRealtyConsultant());
        taCustomer.setStatus(StatusUtils.Normal);
        taCustomer.setCreateDate(LocalDateTime.now());
        taCustomer.setReceiver(taMainUsherRecord.getReceiver());
        taCustomer.setCustomerType(taMainUsherRecord.getCustomerType());
        taCustomer.setCustomerSource(taMainUsherRecord.getCustomerSource());

        return taCustomer;
    }

    private boolean insertCustomer(TaCustomer taCustomer) {
        Integer consultantId = taCustomer.getConsultantId();
        if (null != consultantId && consultantId.intValue() > 0) {
            try {
                notifyConsultant(taCustomer);
            } catch (Exception e) {}
        }

        return taCustomerMapper.insert(taCustomer) > 0;
    }
    
    
    
    /**
     * 推送消息去吧台
     * @param taCustomer
     */
    @Override
    public void sendMessageToBar(TaCustomer taCustomer,Integer personNum) {
        Map<String,Object> map = new HashMap<>();
        map.put("customerId",taCustomer.getCustomerId());
        map.put("customerName",taCustomer.getCustomerName());
        map.put("consultantName",taCustomer.getRealtyConsultant());
        map.put("avatar",taCustomer.getAvatar());
        map.put("visitorNum",personNum);
        map.put("visitorTime",LocalDateTime.now());
        map.put("type",CommConstant.IDENTITY_BARUSHER);
        try {
            WebSocketServer.sendInfo(JSONObject.toJSONString(map), CommConstant.WECHAT_CLIENT_MINIAPP, null);
        } catch (Exception e) {}
    }
    
//    @Override
//    public void notifyManager(TaCustomer taCustomer) {
//        String userIdentity = CommConstant.CUSTOMER_OF_CHANNEL.equals(taCustomer.getCustomerType()) ?
//                CommConstant.IDENTITY_CHANNEL_MANAGER : CommConstant.IDENTITY_CONSULTANT_MANAGER;
//
//
//        //获取所有的置业经理信息并推送消息
//        List<SysUser> sysUserList = sysUserMapper.filterByIdentity(userIdentity);
//        sysUserList.stream().forEach(e -> {
//            String tpl = miniApp.getTpls().getVisitor();
//            String page = CommConstant.PAGE_MY_CUSTOMER + "?id=" + taCustomer.getCustomerId();
//            String openid = e.getOpenid();
//            if (null == openid) {
//                return;
//            }
//            List<Object> data = new ArrayList<Object>(){{
//                add(taCustomer.getCustomerName());
//                add(taCustomer.getPhone());
//                add("您有客户来访, 请及时接待!");
//            }};
//
////            miniApp.sendTPLMessage(tpl, openid, page, data);
//            if (miniApp.sendTPLMessage(tpl, openid, page, data).getCode() == -1) {
//                // 没有 formid 发短信
//                String phone = sysUserMapper.getPhoneByOpenid(openid);
//                if (null != phone) {
//                    smsUtils.sendVisitor(phone, null == taCustomer ? "": StringUtils.ifNull(taCustomer.getCustomerName(), ""));
//                }
//            }
//        });
//    }
    
    @Override
    public void sendMessageAll(TaFirstUsherRecord taFirstUsherRecord, TaCustomer taCustomer, TaMainUsherRecord taMainUsherRecord) {
        Map<String,Object> map = new HashMap<>();
        map.put("type",CommConstant.IDENTITY_ALL_USHER);
        if (null != taFirstUsherRecord){
            map.put("taFirstUsherRecord",taFirstUsherRecord);
        }
        if (null != taCustomer){
            map.put("taCustomer",taCustomer);
        }
        if (null != taMainUsherRecord){
            map.put("taMainUsherRecord",taMainUsherRecord);
        }
        try {
            WebSocketServer.sendInfo(JSONObject.toJSONString(map), CommConstant.WECHAT_CLIENT_MINIAPP, null);
        } catch (Exception e) {}
    }

}
